 ;Jaws For Windows scripts for Adobe Audition
;Version: 20050117-1
	;Authors: Michael Curran and James Teh 

Include "audition.jsh" ;Audition header file
include "audition.jsm" ;Audition messages
include "msaa.jsh"
include "wm.jsh"
Include "HjGlobal.jsh" ; default HJ global variables
Include "hjconst.jsh" ; default HJ constants
Include "common.jsm" ; message file

Globals
string ProductVersion, ;Audition's version
string ProductName, ;Audition's name
;Disabling events:
int DisableMenuModeEvent, ;If true, MenuModeEvent will not run
int DisableFocusChangedEvent, ;If true, FocusChangedEvent will not run
int DisableAutoStartEvent_FirstTime, ;If true, the once-off part of AutoStartEvent will not run
;Toolbar variables:
int tb_index, ;The number of the current toolbar
string tb_list, ;a list of all the toolbars that can be found. updated each time you go to the toolbars 
int tb_buttonIndex, ;The number of the current button
string tb_buttonList, ;A list of all the button coordinates for the current toolbar
int tb_buttons, ;The number of buttons on the current toolbar
;Backup variables:
int OldTypingEcho, ;Holds the old Typing Echo setting
Int OldeditorMode, ;Holds the old editorMode (edit view or track view)
string LastStatusText, ;Holds the last status message. Used to work out if the current status message should be spoken.
;Other vars
Int editorMode, ;Holds the current editorMode (edit view or Track view)
int AutoTrackZoom, ;If true we are allowed to zoom in the track view when it gets focus
handle appWindow, ;Holds the window handle of the main Audition window
handle commandWindow, ;Holds the window that all audition commands are to be sent to
; Verbosity Settings
int SpeakCueLabelsVerbosity, ;If true then cueue labels are spoken when jumpt to
int SpeakStatusVerbosity, ;If false then not as many status messages will be spoken
int PositionalTimesVerbosity, ;If true then certain times will be spoken as "beginning" or "end" etc
int DescriptiveTimesVerbosity, ;If true then times will be said in english (e.g. 3minutes5seconds) 
int NavigateTracksByVerbosity, ;Which fields you want to hear automatically as you move up and down tracks
int AutoStartComplete, ;If true then AutoStartEvent has finished
handle WndTrackNumber, ;The window handle of the track numbers ruler
string TrackFieldList, ;Holds the  list of track fields (in the order you want them to be moved through)
int TrackFieldIndex, ;Holds the number of the current track field
int blockNumber, ;Holds the number of the current block?
; Special markers
string TimeMark1, ;A place to hold a time
string TimeMark2, ;Another place to hold a time
string SelectionMarkerList, ;Holds a list of selections
string ViewMarkerList, ;Holds a list of views
int ExtendedDlgMode,  ; Turns the extended dialog mode on/off
int wasPaused ;Used in the jump cue functions

Void Function AutoStartEvent ()
; Retrieve and store the application version.
; This has to be done each time the scripts start instead of just the first time in case of multiple versions of Audition being run during this instance of JAWS.
let ProductVersion = GetVersionInfoString(GetAppFilePath(),"ProductVersion")
;Also get the name of the program
let ProductName = GetVersionInfoString(GetAppFilePath(),"ProductName")

;jaws generally tries to speak the name of the currently loaded file and not the program title... so:
Say(formatString("%1 %2",ProductName,ProductVersion),OT_APP_NAME)

;Check to see if we can get Audition's main window handle now
if GetWindowClass(GetAppMainWindow(GetFocus())) == classAudition then
let appWindow = GetAppMainWindow(GetFocus())
let commandWindow = getFinalWindow(getFirstChild(appWindow))
Endif

;Only run the following if this is the first time AutoStartEvent has run
if !disableAutoStartEvent_FirstTime then
;Tell the user about hotkeys
Say(FormatString(msgHotkeyReminder),OT_MESSAGE)

;Inicialise some important variables:
let OldTypingEcho = -1 
let AutoTrackZoom = FALSE
let TrackFieldIndex = 1
;Grab all the verbocity settings from the jsi file and set them
LoadSettings()
;Set this variable so that this stuff doesn't run the next time Audition gets focus
let disableAutoStartEvent_FirstTime = TRUE
EndIf
EndFunction

Function SayTrackFieldValue ()
var
string CurrentTrackField,
int x,
handle window,
string value
let CurrentTrackField = StringSegment (TrackFieldList, "|",TrackFieldIndex)
if CurrentTrackField == "name" then
SayUsingVoice(VCTX_PcCursor,GetWindowText(FindDescendantWindow(appWindow,CidTrackName),False),OT_CONTROL_VALUE)
elif CurrentTrackField == "type" then
SayUsingVoice(VCTX_PcCursor,GetTrackType(),OT_CONTROL_VALUE)
elif CurrentTrackField == "blocks" then
SayUsingVoice(VCTX_PcCursor,MoveToAndGetTrackBlockValue(x),OT_CONTROL_VALUE)
; x = block count
; If there are currently no blocks, do not speak block count.
if x > 0 then
SayUsingVoice(VCTX_MESSAGE,FormatString("%1 of %2",IntToString(BlockNumber),IntToString(x)),OT_ITEM_NUMBER)
EndIf
elif CurrentTrackField == "mute" then
let window = findDescendantWindow(appWindow,cidTrackMute)
let value = getTrackfieldState(getDECCState(window))
sayUsingVoice(VCTX_PCCursor,value,OT_CONTROL_NAME)
elif CurrentTrackField == "solo" then
let window = findDescendantWindow(appWindow,cidTrackSolo)
let value = getTrackfieldState(getDECCState(window))
sayUsingVoice(VCTX_PCCursor,value,OT_CONTROL_NAME)
elif CurrentTrackField == "arm" then
let window = findDescendantWindow(appWindow,cidTrackArm)
let value = getTrackfieldState(getDECCState(window))
sayUsingVoice(VCTX_PCCursor,value,OT_CONTROL_NAME)
elif CurrentTrackField == "pan" then
SayUsingVoice(VCTX_PcCursor,GetTrackPanValue(),OT_CONTROL_VALUE)
elif CurrentTrackField == "volume" then
let window = findDescendantWindow(appWindow,cidTrackVolume)
let value = getEndValue(getDECCValue(window))
SayUsingVoice(VCTX_PcCursor,value,OT_CONTROL_VALUE)
elif CurrentTrackField == "playdevice" then
SayUsingVoice(VCTX_PcCursor,GetTrackPlaydeviceValue(),OT_CONTROL_VALUE)
elif CurrentTrackField == "recdevice" then
SayUsingVoice(VCTX_PcCursor,GetTrackRecdeviceValue(),OT_CONTROL_VALUE)
elif CurrentTrackField == "midimap" then
SayUsingVoice(VCTX_PcCursor,GetTrackMidimapValue(),OT_CONTROL_VALUE)
elif CurrentTrackField == "fx" then
SayUsingVoice(VCTX_PcCursor,GetTrackFXValue(),OT_CONTROL_VALUE)
elif CurrentTrackField == "Lock" then
let window = findDescendantWindow(appWindow,cidTrackLock)
let value = getTrackfieldState(getDECCState(window))
sayUsingVoice(VCTX_PCCursor,value,OT_CONTROL_NAME)
elif CurrentTrackField == "EQHigh" then
let window = findDescendantWindow(appWindow,cidTrackEQHigh)
let value = getEndValue(getDECCValue(window))
SayUsingVoice(VCTX_PcCursor,value,OT_CONTROL_VALUE)
elif CurrentTrackField == "EQMid" then
let window = findDescendantWindow(appWindow,cidTrackEQMid)
let value = getEndValue(getDECCValue(window))
SayUsingVoice(VCTX_PcCursor,value,OT_CONTROL_VALUE)
elif CurrentTrackField == "EQLow" then
let window = findDescendantWindow(appWindow,cidTrackEQLow)
let value = getEndValue(getDECCValue(window))
SayUsingVoice(VCTX_PcCursor,value,OT_CONTROL_VALUE)
elif CurrentTrackField == "EQMode" then
SayUsingVoice(VCTX_PcCursor,GetTrackEQMode(),OT_CONTROL_VALUE)
elif CurrentTrackField == "Wet" then
let window = findDescendantWindow(appWindow,cidTrackWet)
let value = getEndValue(getDECCValue(window))
SayUsingVoice(VCTX_PcCursor,value,OT_CONTROL_VALUE)
elif CurrentTrackField == "Dry" then
let window = findDescendantWindow(appWindow,cidTrackDry)
let value = getEndValue(getDECCValue(window))
SayUsingVoice(VCTX_PcCursor,value,OT_CONTROL_VALUE)
Endif
EndFunction

Void Function PriorTrackField ()
if !isTrackTable() || IsToolbar(GetFocus()) || getEditorMode() != editorModeTrack then
executeDefaultScript()
return
Endif
let TrackFieldIndex = TrackFieldIndex - 1
if(TrackFieldIndex == False) then
let TrackFieldIndex = TrackFieldIndex + 1
Endif 
SayTrackFieldHeading()
SayTrackFieldValue()
EndFunction

Void Function NextTrackField ()
if !isTrackTable() || IsToolbar(GetFocus()) || getEditorMode() != editorModeTrack then
executeDefaultScript()
return
Endif
let TrackFieldIndex = TrackFieldIndex + 1
if TrackFieldIndex > (NumberOfStringsInString(TrackFieldList,"|") + 1) then
let TrackFieldIndex = TrackFieldIndex - 1
Endif 
SayTrackFieldHeading()
SayTrackFieldValue()
EndFunction
	
Script Enter ()
if IsToolbar(GetFocus()) && !MenusActive() then
tbActivateButton()
Return
;Are we in the cueue list?
elif (GetControlID(GetFocus()) == CidCueList) &&
IsThisUnderWindow(GetFocus(),GetDockingWindow("Cue List")) then
MoveToHighlightedCue()
elif isTrackTable() then
;We don't want to be pressing enter on a track field *too* fast (might think its a double click)
if IsSameScript() then
Pause()
Endif
ActivateTrackField(False)
else
executeDefaultScript()
Endif
EndScript

Script CtrlEnter ()
if isTrackTable() then
activateTrackField(true)
else
executeDefaultScript()
endIf
EndScript

String Function GetFileName ()
var
String name
if IsWindowDisabled(FindDescendantWindow(appWindow,CidSelectFrom)) then
return msgNoFiles
Endif
let name = GetWindowName(appWindow)
let name = StringTrimTrailingBlanks (name)
let name = stringChopRight (name,16)
if(StringRight(name,LenReadOnlyTitle) == TitleReadOnly) then
let name = StringChopRight(name,11)
Endif
if(name != TitleAuditionNoFiles) then
if !stringIsBlank(name) then
return "\""+name+"\""
else
return msgNoFiles
endIf
else
return msgNoFiles
Endif
EndFunction

Void Function switchFiles ()
var
string Old
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
if(GetFileName() == msgNoFiles) then
SayUsingVoice(VCTX_MESSAGE,msgNoFiles,OT_JAWS_MESSAGE)
return
Endif
let Old = GetWindowName(appWindow) 
typeCurrentScriptKey()
pause()
Pause()
if GetWindowName(appWindow) != Old then 
Say(GetFileName(),OT_DOCUMENT_NAME)
Endif
EndFunction

Script SayCurrentTime ()
if !isTrackTable() &&
!isWaveform() then
executeDefaultScript()
return
Endif
SayUsingVoice(VCTX_MESSAGE,ConvertTimeUser(GetTimeCurrent()),OT_STATUS)
EndScript

Void Function SpeakSelectionOrEditCursor ()
var
string SelectFrom,
string SelectTo,
string SelectLength
if GetFileName() == msgNoFiles then
return
Endif
let SelectFrom = ConvertTimeUser(GetDECCEditboxValue(CidSelectFrom))
let SelectTo = ConvertTimeUser(GetDECCEditboxValue(CidSelectTo))
let SelectLength = ConvertTimeInternal(GetDECCEditboxValue(CidSelectLength),False,PositionalTimesVerbosity)
if !IsSelected() then
SayUsingVoice(VCTX_MESSAGE,SelectFrom,OT_STATUS)
else
SayUsingVoice(VCTX_MESSAGE,FormatString(msgFromToLength,SelectFrom,SelectTo,SelectLength),OT_MESSAGE)
Endif
EndFunction

Void Function ActivateTrackField (int controlState)
var
string CurrentTrackField,
handle trackView,
handle window,
string value
let trackView = findDescendantWindow(appWindow,cidTrackDisp)
let CurrentTrackField = StringSegment (TrackFieldList, "|", TrackFieldIndex)
if CurrentTrackField == "name" then
setFocusToControl(CidTrackName)
elif CurrentTrackField == "blocks" then
OpenBlockInEditView()
elif CurrentTrackField == "mute" then
let window = findDescendantWindow(trackView,cidTrackMute)
let value = getTrackfieldState(toggleDECCState(window,controlState))
sayUsingVoice(VCTX_PCCursor,value,OT_CONTROL_NAME)
elif CurrentTrackField == "solo" then
let window = findDescendantWindow(trackView,cidTrackSolo)
let value = getTrackfieldState(toggleDECCState(window,controlState))
sayUsingVoice(VCTX_PCCursor,value,OT_CONTROL_NAME)
elif CurrentTrackField == "arm" then
if GetTransport() == TransPlay then
SayUsingVoice(VCTX_MESSAGE,MSGMustStop,OT_ERROR)
else
let window = findDescendantWindow(trackView,cidTrackArm)
let value = getTrackfieldState(toggleDECCState(window,controlState))
sayUsingVoice(VCTX_PCCursor,value,OT_CONTROL_NAME)
Endif
elif CurrentTrackField == "pan" then
InputTrackPanValue()
elif CurrentTrackField == "volume" then
InputTrackVolumeValue()
elif CurrentTrackField == "playdevice" then
OpenTrackPlayDeviceSettings()
elif CurrentTrackField == "recdevice" then
OpenTrackRecDeviceSettings()
elif CurrentTrackField == "midimap" then
OpenTrackMidiMapSettings()
elif CurrentTrackField == "fx" then
OpenTrackFXSettings()
elif CurrentTrackField == "lock" then
let window = findDescendantWindow(trackView,cidTrackLock)
let value = getTrackfieldState(toggleDECCState(window,controlState))
sayUsingVoice(VCTX_PCCursor,value,OT_CONTROL_NAME)
elif CurrentTrackField == "EQMode" then
SayUsingVoice(VCTX_PcCursor,ChangeTrackEQMode(),OT_CONTROL_VALUE)
Endif
EndFunction


Script auditionExtendSelectionToBeginning ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
TypeCurrentScriptKey ()
pause()
speakSelectionOrEditCursor()
EndScript

Script auditionExtendSelectionToEnd ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
typeCurrentScriptKey()
pause()
speakSelectionOrEditCursor()
EndScript

Script SayView ()
var
string ViewFrom,
string ViewTo,
string ViewLength
if !isTrackTable() && !isWaveform() then
executeDefaultScript()
return
Endif
let ViewFrom = ConvertTimeUser(GetDECCEditboxValue(CidViewFrom))
let ViewTo = ConvertTimeUser(GetDECCEditboxValue(CidViewTo))
let ViewLength = ConvertTimeInternal(GetDECCEditboxValue(CidViewLength),False,PositionalTimesVerbosity)
SayUsingVoice(VCTX_MESSAGE,FormatString(msgFromToLength,ViewFrom,ViewTo,ViewLength),OT_MESSAGE)
EndScript

Script MoveToSVCtrls ()
;if Transport == 	TransPlay then;Say(msgMustStop,OT_MESSAGE)
;Return
;Endif
if GetControlID(GetParent(GetParent(GetFocus()))) != CidSVCtrls  then
setFocusToControl(CidSelectFrom)
Endif
EndScript

Script ToggleExtendedDlgMode ()
if (ExtendedDlgMode == 0) then
	let ExtendedDlgMode = 1
	Say ("The extended dialog mode is now on", OT_STATUS, 0)
else
	let ExtendedDlgMode = 0
	Say ("The extended dialog mode is now off", OT_STATUS)
EndIf
EndScript

Script Tab ()
if (ExtendedDlgMode == 1) then
if (GetWindowClass (GetParent (GetCurrentWindow ()))=="#32770") then
If (GetWindowClass(GetCurrentWindow()) == "#32770") then
	SetFocus (GetFirstChild(GetCurrentWindow()))
	Return
ElIf (GetWindowClass (GetNextWindow (GetCurrentWindow())) == "Static") then
	SetFocus (GetNextWindow (GetNextWindow (GetCurrentWindow ())))
	Return
ElIf (GetCurrentWindow () == 0) then
	SetFocus (GetFirstWindow(GetParent(GetCurrentWindow())))
	Return
Else
	SetFocus (GetNextWindow (GetCurrentWindow()))
	Return
EndIf
EndIf
EndIf

If IsToolbar(GetFocus()) then
SayCurrentScriptKeyLabel()
tbNextButton()
tbAnnounceButton()
Return
Endif
if isTrackTable() then
NextTrackField()
Return
else
executeDefaultScript()
Endif
EndScript

Script ShiftTab ()
if (ExtendedDlgMode == 1) then
if (GetWindowClass (GetParent (GetCurrentWindow ()))=="#32770") then
If (GetWindowClass(GetCurrentWindow()) == "#32770") then
	SetFocus (GetFirstChild(GetCurrentWindow()))
	Return
ElIf (GetWindowClass (GetPriorWindow (GetCurrentWindow())) == "Static") then
	SetFocus (GetPriorWindow (GetPriorWindow (GetCurrentWindow ())))
	Return
ElIf (GetCurrentWindow () == 0) then
	SetFocus (GetFirstWindow(GetParent(GetCurrentWindow())))
	Return
Else
	SetFocus (GetPriorWindow (GetCurrentWindow()))
	Return
EndIf
EndIf
EndIf

If IsToolbar(GetFocus()) then
SayCurrentScriptKeyLabel()
tbPriorButton()
tbAnnounceButton()
Return
Endif
if isTrackTable() then
PriorTrackField()
Return
else
executeDefaultScript()
Endif
EndScript

Script Escape ()
if UserBufferIsActive () then
UserBufferClear ()
UserBufferDeactivate ()
FocusChangedEvent(GetFocus(),0)
Return
Endif
if GetWindowClass(GlobalPrevReal) == ClassFloatingDock then
{Alt+f4}
Pause()
Return
Endif
if IsInSVCtrls(GetFocus()) || IsToolbar(GetFocus()) then
;TypeCurrentScriptKey()
SayCurrentScriptKeyLabel()
PutFocusToMainArea()
if GetTransport() != TransStop then
if editorMode == editorModeTrack then
sayTrackTable()
elif editorMode == editorModeWave then
sayWaveform()
elif editorMode == editorModeCD then
SayWindowTypeAndText (getFocus())
endIf
Endif
Return
Endif
if !isTrackTable() && !isWaveform() then
executeDefaultScript()
Return
Endif
if (GetDECCEditboxValue(CidSelectTo) == cscNull) then
SayUsingVoice(VCTX_MESSAGE,msgNoSelection,OT_STATUS)
return
elif (GetTransport() == TransPlay) then
SayUsingVoice(VCTX_MESSAGE,msgMustStop,OT_ERROR)
Return
Endif
; Have to do this so that pressing escape to cancel the list box doesn't cause an unknown function call.
; This is because the list box is presented within this script, and so pressing escape within the list box attempts to run the script from within itself.
; A little dodgy, but better than the alternative.
ScheduleFunction("ClearSelection",0)
EndScript

Function ClearSelection ()
var
int index,
string time,
string list

let index = DlgSelectItemInList(J_ListClearSelection,J_TitleClearSelectionList,FALSE)
if(index == 0) || (index == 1) then
return
elif(index == 2) then
let time = GetDECCEditboxValue(CidSelectFrom)
{escape}
ChangeDECCEditboxValue(CidSelectFrom,Time)
elif(index == 3) then
let time = GetDECCEditboxValue(CidSelectTo)
{escape}
ChangeDECCEditboxValue(CidSelectFrom,Time)
elif(index == 4) then
{escape}
Endif
EndFunction

Int Function IsTimeBeginning (string time)
if StringMatch(time,"00:00:00:00") ||
StringMatch(time,"0:00.000") ||
StringMatch(time,"1:1.00") then
Return True
else
Return False
Endif
EndFunction
	
String Function GetTrackType ()
var
handle window
SaveCursor()
if MoveToAndGetTrackBlockValue(0) == msgEmpty then
Return msgEmpty
else
if IsWindowVisible(FindDescendantWindow(appWindow,CidTrackMap)) then
Return msgTrackTypeMidi
else
Return msgTrackTypeAudio
Endif
Endif
sayInteger(stringRef("hello"))
EndFunction

Int Function getEditorMode ()
if IsWindowVisible(FindDescendantWindow(appWindow,cidWaveDisp)) then
Return editorModeWave 
elif IsWindowVisible(FindDescendantWindow(appWindow,cidTrackDisp)) then
Return editorModeTrack
elif IsWindowVisible(FindDescendantWindow(appWindow,cidCDDisp)) then
return editorModeCD 
Endif
EndFunction

String Function GetTimeCurrent ()
Return GetWindowText(FindDescendantWindow (appWindow,CidTimeCurrent),False)


EndFunction

String Function GetTransport ()
var
string x
SaveCursor()
InvisibleCursor()
MoveToWindow(FindDescendantWindow (appWindow,CidStatusbar))
JawsHome()
let x = GetChunk()
if (x == TransRecord) ||
(x == TransPlay) then
return x
else
return TransStop
Endif
EndFunction

HANDLE Function GetTrackNumberWindow ()
var
handle Window
let window = findDescendantWindow(appWindow,cidTrackDisp)
let Window = FindDescendantWindow(Window,CidTracksRuler)
Return Window
EndFunction


Script GotoTrack ()
var
string input,
int Number,
int Current,
int ok,
int StatusCount
if !isTrackTable() || IsToolbar(GetFocus()) || getEditorMode() != editorModeTrack then
executeDefaultScript()
Return
Endif
while(ok == False) 
if !InputBox(msgGoToTrack_DialogText,msgGoToTrack_DialogTitle,input) then
Return
Endif
StopSpeech ()
let Number = StringToInt(input)
if(Number < 1) || (Number > MaxTracks) then
MessageBox (msgGoToTrack_BadNumber)
StopSpeech ()
else
let ok = True
Endif
EndWhile
let AutoTrackZoom = False
let DisableFocusChangedEvent = True
let Current = StringToInt(GetTrackNumber())
SayUsingVoice(VCTX_MESSAGE,FormatString(msgGoToTrack_GoingToTrack,input),OT_MESSAGE)
Pause()
while(Number < Current		 )
let StatusCount = StatusCount + 1
sendMessage(commandWindow,WM_command,command_trackUp)
let Current = Current - 1
Pause()
if StatusCount == 10 then
SayUsingVoice(VCTX_PcCursor,IntToString(Current)+"...",OT_SCREEN_MESSAGE)
let StatusCount = 0
Endif
Endwhile
while(Number > Current)
let StatusCount = StatusCount + 1
sendMessage(commandWindow,WM_command,command_trackDown)
let Current = Current + 1
Pause()
if StatusCount == 10 then
SayUsingVoice(VCTX_PcCursor,IntToString(Current)+"...",OT_SCREEN_MESSAGE)
let StatusCount = 0
Endif
Endwhile
While(GetTrackNumber() != Number)
Pause()
Endwhile
SpeechOn()
let AutoTrackZoom = True
let DisableFocusChangedEvent = False
SpeechOff()
FocusChangedEvent(GetFocus(),GetFocus())
Pause()
SpeechOn()
SayTrackNavigation()
EndScript

String Function GetFileFormat ()
SaveCursor()
InvisibleCursor()
MoveToWindow(FindDescendantWindow (appWindow,CidStatusbar))
JawsHome()
NextChunk()
Return GetChunk()
EndFunction

String Function GetFileSize ()
SaveCursor()
InvisibleCursor()
MoveToWindow(FindDescendantWindow (appWindow,CidStatusbar))
JawsHome()
NextChunk()
NextChunk()
Return GetChunk()
EndFunction


String Function GetFileLength ()
SaveCursor()
InvisibleCursor()
MoveToWindow(FindDescendantWindow (appWindow,CidStatusbar))
JawsHome()
NextChunk()
NextChunk()
NextChunk()
Return StringTrimLeadingBlanks(GetChunk())
EndFunction

Void Function PutFocusToMainArea ()
var
handle waveDisp,
handle trackDisp,
handle window
if getEditorMode() == editorModeTrack then
let trackDisp = findDescendantWindow(appWindow,cidTrackDisp)
let window = FindDescendantWindow(trackDisp,CidTrackCtrl)
else
let waveDisp = findDescendantWindow(appWindow,cidWaveDisp)
let window = FindDescendantWindow(WaveDisp,CidWaveCtrl)
Endif
if getFocus() != window then
setFocus(window)
sendMessage(appWindow,wm_lbuttonDown)
sendMessage(appWindow,wm_lbuttonUp)
endIf
EndFunction

Script SayCurrentLine ()
if !isTrackTable() || IsToolbar(GetFocus()) || getEditorMode() != editorModeTrack then
executeDefaultScript()
Return
Endif
SayTrackNavigation()
EndScript

Void Function AutoFinishEvent ()
RestoreTypingEcho()

;Was Audition actually cloased?
if !GetWindowName(appWindow) then
let disableAutoStartEvent_FirstTime = False
let LastStatusText = ""
Endif
let appWindow = StringToHandle (CSCNull)
EndFunction

Int Function getDockingWindowState (handle Window)
var
int out
if !window then
let out = closed
elif !IsThisUnderWindow(GetFocus(),Window) then
let out = lost 
else 
let out = Open
Endif
Return out
EndFunction

Void Function NewTextEvent (handle Window, string Text, int Attributes, int FGColor, int BGColor, int Echo, string Frame)
var
string StatusbarText
;See if this text is a status message, and see if we are aloud to speak status messages.
;For some odd reason, the statusbar is refreshed every time Audition gets focus 
;(therefor we need to make sure its a new status message, not the same one as last time)
;We will only look at them if we  are under the Audition main window.
;This makes sure they won't be spoken if we are in the Open dialog etc
if (!appWindow || (GetRealWindow(GetFocus()) == appWindow)) && 
(GetControlID(window) == CidStatusbar) && SpeakStatusVerbosity && !StringMatch(text,LastStatusText) then
;Lets work out which status message it is, and speak it accordingly.
;We will also update LastStatusText to keep track of new messages
if StringContains(Text,"Opened") then 
SayUsingVoice(VCTX_PcCursor,Text,OT_SCREEN_MESSAGE)
let LastStatusText = text
elif StringContains(Text,"Completed") then
SayUsingVoice(VCTX_PcCursor,Text,OT_SCREEN_MESSAGE)
let LastStatusText = text
elif StringContains(Text,"Copyright") then
SayUsingVoice(VCTX_PcCursor,Text,OT_SCREEN_MESSAGE)
let LastStatusText = text
elif text == TransPlay then
;SayUsingVoice(VCTX_PcCursor,Text,OT_SCREEN_MESSAGE)
let LastStatusText = text
elif text == TransRecord then
;SayUsingVoice(VCTX_PcCursor,Text,OT_SCREEN_MESSAGE)
let LastStatusText = text
elif (Text == TransStop) then
SayUsingVoice(VCTX_PcCursor,Text,OT_SCREEN_MESSAGE)
SpeakSelectionOrEditCursor()
let LastStatusText = text
Endif
Return
Endif
NewTextEvent(Window,Text,Attributes,FGColor,BGColor,Echo,Frame)
EndFunction

Void Function setFocusToControl (int ControlID)
SaveCursor()
JawsCursor()
MoveToWindow(FindDescendantWindow(appWindow,ControlID))
LeftMouseButton()
RestoreCursor()
EndFunction

Int Function IsThisUnderWindow (handle Child, handle Parent)
while !((child == parent) || (GetWindowHierarchyY(child) == 1))
let child = getParent(child)
EndWhile
return (parent == child)
EndFunction

Script ScriptFileName ()
ScriptAndAppNames(msgAdobeAudition)
EndScript

Int Function StringMatch (string StringA, string StringB)
; To determine whether two strings truly match, we have to use StringLength comparison  and StringContains because of stupid bugs in
; JAWS's == comparison operator,which seems to check only the start of strings and exhibits other seriously dodgy behaviour.
; (I never knew that "2.0" == "2.1"!)
if (StringLength(StringA) == StringLength(StringB)) && StringContains(StringA,StringB) then
return true
else
return false
EndIf
EndFunction

Int Function NumberOfItemsInGroup ()
var
string pos
let pos = PositionInGroup()
if (pos == "0 of 0") || (pos == cscNull) then
return 0
else
return StringSegment(pos,cscSpace,3)
EndIf
EndFunction

Void Function JumpCueHelper (int direction)
var
int NumCues,
int count,
string foundCue,
string cueName,
string cueDescription,
string currentTime,
string cueTime,
handle CueListWindow,
handle cueDialog,
object CuelistObject,
int CuelistItemId

if (GetTransport() == TransPlay) && !wasPaused then
let wasPaused = true
sendMessage(commandWindow,WM_command,command_pause)
pause()
pause()
EndIf
if direction == S_NEXT then
sendMessage(commandWindow,wm_command,command_jumpToNextCue)
else
sendMessage(commandWindow,wm_command,command_jumpToPreviousCue)
if isSameScript() then
sendMessage(commandWindow,wm_command,command_jumpToPreviousCue)
endIf
endIf
pause()
let currentTime = GetTimeCurrent()
if SpeakCueLabelsVerbosity then
let DisableFocusChangedEvent = True
; In the cue list, find the cue beginning with the current time.
;First make sure there's no hidden docking windows
configureAuditionWindows ()
sendMessage(commandWindow,WM_command,command_toggleCueListWindow)
if getWindowName(getRealWindow(getFocus())) != "Cue List" then
sayString("Error: cue list")
let disableFocusChangedEvent = false
return
endIf
;Use MSAA to find the correct cue
let cueDialog = getFocus()
let cueListWindow = findDescendantWindow(cueDialog,cidCueList)
let cueListObject = getObjectFromEvent(cueListWindow,-4,0,cueListItemID)
let NumCues = CuelistObject.AccChildCount() 
let count = 1
while !(foundCue || (count >= NumCues))
let cueName = cueListObject.accName(count)
let cueDescription = cueListObject.accDescription(count) 
; Order of fields is different between Audition 1.0 and 1.5.
if stringContains(cueDescription, "Begin") == 1 then
; Audition 1.0
let cueTime = stringSegment(cueDescription, ",", 1)
else
let cueTime = stringSegment(cueDescription, ",", 2)
endIf
let cueTime = stringStripAllBlanks(stringChopLeft(cueTime, stringContains(cueTime, ":")))
if stringMatch(cueTime,currentTime) then
; Found it; grab the name of the cue.
SayString(CuelistObject.AccName(count)) 
EndIf
let count = count + 1
EndWhile
sendMessage(commandWindow,WM_COMMAND,command_toggleCueListWindow)
let DisableFocusChangedEvent = False
EndIf ;end of SpeakCueLabels
if wasPaused then ; If playing before the jump
if ConvertTimeInternal(GetTimeCurrent(), true, false) == msgEnd then
; If we are at the end, playback has stopped and there is no need to unpause.
let wasPaused = false
else
; We're playing, but we're not at the end.
PerformScript SayCurrentTime()
EndIf
else
PerformScript SaySelection()
EndIf
if IsSelected() then
PerformScript SaySelection()
EndIf
if wasPaused then
sendMessage(commandWindow,WM_command,command_pause)
let wasPaused = false
EndIf
EndFunction

Script auditionJumpToNextCue ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
jumpCueHelper(S_NEXT)
EndScript

Script auditionJumpToPreviousCue ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
jumpCueHelper(S_PRIOR)
EndScript

Script AdjustJAWSVerbosity ()
var
int OldSpeakCueLabelsVerbosity,
int OldSpeakStatusVerbosity,
int OldPositionalTimesVerbosity,
int OldDescriptiveTimesVerbosity,
int OldNavigateTracksByVerbosity
; Store current values of settings to check if settings need to be saved.
let OldSpeakCueLabelsVerbosity = SpeakCueLabelsVerbosity
let OldSpeakStatusVerbosity = SpeakStatusVerbosity
let OldPositionalTimesVerbosity = PositionalTimesVerbosity

let OldDescriptiveTimesVerbosity = DescriptiveTimesVerbosity
let OldNavigateTracksByVerbosity = NavigateTracksByVerbosity
JawsVerbosityCore (J_ListVerbosity)
; If settings have changed, save settings; otherwise, don't bother.
if (SpeakCueLabelsVerbosity != OldSpeakCueLabelsVerbosity) ||
 (SpeakStatusVerbosity != OldSpeakStatusVerbosity) ||
 (PositionalTimesVerbosity != OldPositionalTimesVerbosity) ||
 (DescriptiveTimesVerbosity != OldDescriptiveTimesVerbosity) ||
 (NavigateTracksByVerbosity != OldNavigateTracksByVerbosity) then
SaveSettings()
EndIf
EndScript

String Function ToggleSpeakCueLabelsVerbosity (int RetCurVal)
if !RetCurVal then
; Update the value.
let SpeakCueLabelsVerbosity = not SpeakCueLabelsVerbosity
EndIf
if SpeakCueLabelsVerbosity then
return msgOn
else
return msgOff
EndIf
EndFunction

String Function ToggleSpeakStatusVerbosity (int RetCurVal)
if !RetCurVal then
; Update the value.
let SpeakStatusVerbosity = not SpeakStatusVerbosity
EndIf
if SpeakStatusVerbosity then
return msgOn
else
return msgOff
EndIf
EndFunction

String Function TogglePositionalTimesVerbosity (int RetCurVal)
if !RetCurVal then
; Update the value.
let PositionalTimesVerbosity = not PositionalTimesVerbosity
EndIf
if PositionalTimesVerbosity then
return msgOn
else
return msgOff
EndIf
EndFunction

String Function ToggleDescriptiveTimesVerbosity (int RetCurVal)
if !RetCurVal then
; Update the value.
let DescriptiveTimesVerbosity = not DescriptiveTimesVerbosity
EndIf
if DescriptiveTimesVerbosity then
return msgOn
else
return msgOff
EndIf
EndFunction
String Function ToggleNavigateTracksByVerbosity (int RetCurVal)
if !RetCurVal then
; Update the value.
let NavigateTracksByVerbosity = NavigateTracksByVerbosity + 1
if NavigateTracksByVerbosity > CfgNavigateTracksByNumberAndName then
let NavigateTracksByVerbosity = CfgNavigateTracksByNumber
endIf
EndIf
return StringSegment(MSGListNavigateTracksBy,"|",NavigateTracksByVerbosity + 1)
EndFunction

void Function LoadSettings ()
; Read the settings from the jsi file.
let SpeakCueLabelsVerbosity = IniReadInteger(JsiSectAppVerbosity,JsiKeySpeakCueLabelsVerbosity,true,JsiFileName)
let SpeakStatusVerbosity = IniReadInteger(JsiSectAppVerbosity,JsiKeySpeakStatusVerbosity,true,JsiFileName)
let PositionalTimesVerbosity = IniReadInteger(JsiSectAppVerbosity,JsiKeyPositionalTimesVerbosity,true,JsiFileName)
let DescriptiveTimesVerbosity = IniReadInteger(JsiSectAppVerbosity,JsiKeyDescriptiveTimesVerbosity,true,JsiFileName)
let NavigateTracksByVerbosity = IniReadInteger(JsiSectAppVerbosity,JsiKeyNavigateTracksByVerbosity,CfgNavigateTracksByNumber,JsiFileName)
EndFunction

Void Function SaveSettings ()
var
int result
; Only do the let result bit for the first IniWrite.
let result = IniWriteInteger(JsiSectAppVerbosity,JsiKeySpeakCueLabelsVerbosity,SpeakCueLabelsVerbosity,JsiFileName)
IniWriteInteger(JsiSectAppVerbosity,JsiKeySpeakStatusVerbosity,SpeakStatusVerbosity,JsiFileName)
IniWriteInteger(JsiSectAppVerbosity,JsiKeyPositionalTimesVerbosity,PositionalTimesVerbosity,JsiFileName)
IniWriteInteger(JsiSectAppVerbosity,JsiKeyDescriptiveTimesVerbosity,DescriptiveTimesVerbosity,JsiFileName)
IniWriteInteger(JsiSectAppVerbosity,JsiKeyNavigateTracksByVerbosity,NavigateTracksByVerbosity,JsiFileName)
if result then
SayFormattedMessage(ot_jaws_message,msgSettingsSaved,cMsgSilent)
else
SayFormattedMessage(ot_jaws_message,msgSettingsNotSaved)
EndIf
EndFunction

Script HotKeyHelp ()
If UserBufferIsActive () then
UserBufferDeactivate ()
EndIf
if GetRealWindow(GetFocus()) == appWindow then
if IsToolbar(GetFocus()) then
SayUsingVoice(VCTX_MESSAGE,"Help for toolbars not yet wrritten.",OT_JAWS_MESSAGE)
return
EndIf
; This help applies to both waveform and multitrack views.
SayFormattedMessage (ot_user_buffer, msgHotKeyHelpWave)
if getEditorMode() == editorModeTrack then
UserBufferAddText(cScBufferNewLine)
SayFormattedMessage (ot_user_buffer, msgHotKeyHelpTrack)EndIf
else
; JAWS default hotkey help. This will be called in dialog boxes, etc.
PerformScript HotKeyHelp()
EndIf
EndScript

Void Function SetGlobalFocusVars (handle New)
let GlobalPrevReal = GetRealWindow(New)
let GlobalPrevRealName = GetWindowName(GetRealWindow(New))
let GlobalPrevApp = GetAppMainWindow(New)
let GlobalPrevFocus = New
Let GlobalWasHjDialog = InHjDialog ()
EndFunction

String Function GetTrackNumber ()
refreshWindow(wndTrackNumber)
Return GetWindowText(wndTrackNumber,false)
EndFunction

Script RRFFSpeed ()
var
handle TransportControls
let TransportControls = FindDescendantWindow(appWindow,CidTransportControls)
RefreshWindow(TransportControls)
pause()
SaveCursor()
InvisibleCursor()
MoveToWindow(TransportControls)
SetRestriction (RestrictWindow)
JawsPageDown()
JawsHome()
NextWord()
RouteJawsToInvisible()
SetRestriction (RestrictAppWindow)
RightMouseButton()
PcCursor()
Pause()
EndScript

Script auditionSetSelectFrom ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
sendMessage(commandWindow,wm_command,command_selectFromCurrent)
pause()
sayUsingVoice(VCTX_MESSAGE,formatString(msgSelectedFrom,ConvertTimeUser(getDECCEditBoxValue(cidSelectFrom))),OT_MESSAGE)
EndScript

Script auditionSetSelectTo ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
sendMessage(commandWindow,wm_command,command_selectToCurrent)
pause()
speakSelectionOrEditCursor()
EndScript

Void Function SilenceTypingEcho ()
if OldTypingEcho == -1 then
let OldTypingEcho = GetJCFOption (opt_typing_echo)
SetJCFOption (OPT_Typing_Echo, 0)
Endif
EndFunction

Function RestoreTypingEcho ()
if OldTypingEcho != -1 then
SetJCFOption (OPT_Typing_Echo, OldTypingEcho)
let OldTypingEcho = -1
EndIf
EndFunction

Void Function SayTime ()
PerformScript SaySelection()
EndFunction

String Function ConvertTimeInternal (string Time, int PositionalTimesVerbosity, int DescriptiveTimesVerbosity)
var
string TimeFormat

; Audition adds trailing spaces to the time fields with certain display time formats, so these must be removed for comparison.
let Time = StringStripAllBlanks(Time)
if PositionalTimesVerbosity && (time == GetFileLength()) then
Return msgEnd
EndIf

; Determine the time format.
let TimeFormat = GetTimeFormatType()
if TimeFormat == "hms" then
return ConvertTimeDecimal(time,PositionalTimesVerbosity,DescriptiveTimesVerbosity)
elif TimeFormat == "bpm" then
return ConvertTimeBarsBeats(time,PositionalTimesVerbosity,DescriptiveTimesVerbosity)
elif TimeFormat == "fps" then
return ConvertTimeHMSF(time,PositionalTimesVerbosity,DescriptiveTimesVerbosity)
elif TimeFormat == "mpl" then
return ConvertTimeSamples(time,PositionalTimesVerbosity,DescriptiveTimesVerbosity)
EndIf
EndFunction

Int Function isTrackTable ()
if (GetControlID(getFocus()) == CidTrackCtrl) && 
(editorMode == editorModeTrack) &&
!menusActive() &&
!UserBufferIsActive() &&
IsPCCursor () then
Return True
else
Return False
Endif
EndFunction

Int Function isWaveform ()
if (GetControlID(getFocus()) == CidWaveCtrl) && 
(editorMode == editorModeWave) &&
!menusActive() &&
!UserBufferIsActive() &&
IsPCCursor () then
Return True
else
Return False
Endif
EndFunction

Int Function toggleDECCState (handle window, int controlState)
var
int safetyCount,
int oldState,
int newState,
int finished
let oldState = getDECCState(window) 
if (oldState == CTRL_DISABLED) ||
(oldState == CTRL_GRAYED) then
return oldState
endIf
if ControlState then
sendMessage(window,WM_LBUTTONDOWN,MK_CONTROL,0)
sendMessage(Window,WM_LBUTTONUP,MK_CONTROL,0)
else
sendMessage(window,WM_LBUTTONDOWN,0,0)
sendMessage(Window,WM_LBUTTONUP,0,0)
Endif
while !finished && (SafetyCount != 20)
let newState = getDECCState(window)
if newState != oldState then
let finished = true
Endif
let SafetyCount = SafetyCount + 1
EndWhile
if SafetyCount == 20 then
SayUsingVoice(VCTX_MESSAGE,msgError,OT_ERROR)
Endif
return newState
EndFunction

String Function GetTrackPanValue ()
var
string Out,
handle Window
let Window = FindDescendantWindow (appWindow, CidTrackPan)
let out = GetWindowText(Window,False)
if StringContains (out,"Pan ") then
Return StringChopLeft(out,4) 
else
return out 
Endif
EndFunction

String Function ChangeTrackPanValue (int direction)
var
handle Window
let Window = FindDescendantWindow(appWindow,CidTrackPan)
if !IsWindowVisible(Window) then
Return msgHidden
Endif
SaveCursor()
JawsCursor()
MoveToWindow(Window)
DragMouse(direction,2,13)
;MoveToWindow(FindDescendantWindow(appWindow,CidTrackName))
;RightMouseButton()
Return GetTrackPanValue()
EndFunction

Void Function InputTrackPanValue ()
var
handle Window,
string tmp,
string NewValue
let NewValue = IntToString(ConvertLeftToNegative(GetTrackPanValue()))
let Window = FindDescendantWindow (appWindow, CidTrackPan)
if !IsWindowVisible(Window) then
SayUsingVoice(VCTX_MESSAGE,msgHidden,OT_ERROR)Return
Endif
InputBox ("Pan:", "TrackPan", NewValue)
SpeechOff()
let DisableFocusChangedEvent = True
JawsCursor()
MoveToWindow(Window)
RightMouseButton()
Pause()
PcCursor()
{Backspace}
TypeString(NewValue)
{Enter}
Pause()
SetGlobalFocusVars(GetFocus())
SpeechOn()
let DisableFocusChangedEvent = False
Return
EndFunction

Int Function ZoomTrackview ()
var
handle TracksRuler,
int safetyCount
let TracksRuler = FindDescendantWindow(FindDescendantWindow(appWindow,cidTrackDisp),CidTracksRuler)
if StringContains(GetWindowText(TracksRuler,False),cscSpace) then
SpeechOff()
let safetyCount = 0
while StringContains(GetWindowText(TracksRuler,False),cscSpace)
if safetyCount <= 20 then
let safetyCount = safetyCount + 1
else
sayUsingVoice(VCTX_MESSAGE,"Couldn't zoom in track view!",OT_MESSAGE)
return false
endIf
sendMessage(commandWindow,WM_COMMAND,command_zoomInVertically,0)
Pause()
Endwhile
sendMessage(commandWindow,WM_command,command_trackUp)
Pause()
SpeechOn()
Endif
return true
EndFunction

Void Function InputTrackVolumeValue ()
var
handle Window,
string tmp,
string NewValue
let window = findDescendantWindow(appWindow,cidTrackVolume)
let NewValue = getEndValue(getDECCValue(window))
let Window = FindDescendantWindow (appWindow, CidTrackVolume)
if !IsWindowVisible(Window) then
SayUsingVoice(VCTX_MESSAGE,msgHidden,OT_ERROR)
Return
Endif
InputBox ("Volume:","Track volume",NewValue)
let DisableFocusChangedEvent = True
SpeechOff()
JawsCursor()
MoveToWindow(Window)
RightMouseButton()
Pause()
PcCursor()
{Backspace}
TypeString(NewValue)
{Enter}
Pause()
SetGlobalFocusVars(GetFocus())
let DisableFocusChangedEvent = False
SpeechOn()
Return
EndFunction

String Function GetTrackPlaydeviceValue ()
var
string out,
handle window
let Window = FindDescendantWindow(appWindow,CidTrackPlaydevice)
if !IsWindowVisible(Window) then
Return msgHidden
Endif
Return GetWindowText(Window,False)
EndFunction


Void Function OpenTrackPlayDeviceSettings ()
var
handle window
let Window = FindDescendantWindow(appWindow,CidTrackPlaydevice)
if !IsWindowVisible(Window) then
SayUsingVoice(VCTX_MESSAGE,msgHidden,OT_ERROR)
Endif
JawsCursor()
MoveToWindow(Window)
LeftMouseButton()
PcCursor()
EndFunction


String Function GetTrackRecdeviceValue ()
var
string out,
handle window
let Window = FindDescendantWindow(appWindow,CidTrackRecdevice)
if !IsWindowVisible(Window) then
Return msgHidden
Endif
let out = GetWindowText(Window,False)
 let out = StringSegment (out, cscSpace, 2)
Return out
EndFunction

Void Function OpenTrackRecDeviceSettings ()
var
handle window
let Window = FindDescendantWindow(appWindow,CidTrackRecdevice)
if !IsWindowVisible(Window) then
SayUsingVoice(VCTX_MESSAGE,msgHidden,OT_ERROR)
Endif
JawsCursor()
MoveToWindow(Window)
LeftMouseButton()
PcCursor()
EndFunction


String Function GetTrackMidimapValue ()
var
string Out,
handle window
let Window = FindDescendantWindow(appWindow,CidTrackMap)
if !IsWindowVisible(Window) then
SayUsingVoice(VCTX_MESSAGE,msgHidden,OT_ERROR)
Return
Endif
let out = GetWindowText(Window,False)
 let out = StringSegment (out, cscSpace, 2)

if StringIsBlank (Out) then
Return "Default"
Endif
let out = "\""+out+"\""
Return out
EndFunction

Void Function OpenTrackMidiMapSettings ()
var
handle window
let Window = FindDescendantWindow(appWindow,CidTrackMap)
if !IsWindowVisible(Window) then
SayUsingVoice(VCTX_MESSAGE,msgHidden,OT_ERROR)
Return
Endif
JawsCursor()
MoveToWindow(Window)
LeftMouseButton()
PcCursor()
EndFunction

String Function GetTrackFXValue ()
var
string out,
int state,
handle window
let Window = FindDescendantWindow (appWindow,CidTrackFX)
let state = GetDECCState(window)
if state == CTRL_none then
return msgNone
elif state == CTRL_pressed then
return "\""+GetWindowText(Window,False)+"\""
else
return getTrackFieldState(state) 
Endif
EndFunction

Void Function OpenTrackFXSettings ()
var
string color,
handle window
SaveCursor()
JawsCursor()
let Window = FindDescendantWindow (appWindow,CidTrackFX)
if !IsWindowVisible(Window) then
SayUsingVoice(VCTX_MESSAGE,msgHidden,OT_ERROR)
Return
Endif
MoveToWindow(Window)
if IsWindowDisabled(window) then
SayUsingVoice(VCTX_MESSAGE,msgDisabled,OT_ERROR)
Return
Endif
LeftMouseButton()
EndFunction

Void Function OpenBlockInEditView ()
var
handle Window

if MoveToAndGetTrackBlockValue(0) == msgEmpty then
SayUsingVoice(VCTX_MESSAGE,msgEmpty,OT_ERROR)
return
EndIf
SaveCursor()
InvisibleCursor()
;if GetColorName(GetColorBackground()) == "050050050" then
if StringContains(GetChunk(),".mid") then
SayUsingVoice(VCTX_MESSAGE,"This file can not be edited",OT_ERROR)
Return
Endif
RouteJawsToInvisible()
LeftMouseButton()
LeftMouseButton()
Pause()
PcCursor()
Return
EndFunction

String Function changeDECCEditboxValue (int Cid, string NewValue)
var
handle Window
SaveCursor()
SpeechOff()
setFocusToControl(Cid)
TypeString(NewValue)
{Enter}
PutFocusToMainArea()
Pause()
SpeechOn()
EndFunction

String Function scrollDECCSpinBox (handle window, int motion)
var
string x
saveCursor()
jawsCursor()
moveToWindow(window)
leftMouseButtonLock()
if motion == SB_lineUp then
mouseRight(2)
elif motion == SB_pageUp then
mouseRight(11)
elif motion == sb_lineDown then
mouseLeft(2)
elif motion == sb_pageDown then
mouseLeft(11)
endIf
leftMouseButtonLock()
restoreCursor()
pause()
pcCursor()
SpeechOn()
EndFunction

Int Function NumberOfStringsInString (string MainString, string SearchString)
var
int MainStringLength,
int SearchStringLength,
int count,
int x
let MainStringLength = StringLength(MainString)
let SearchStringLength = StringLength(SearchString)
let x = 1
while (x <= MainStringLength)
if StringLeft (MainString,SearchStringLength) == SearchString then
let Count = Count + 1
Endif
let MainString = StringChopLeft(MainString,1)
let x = x + 1
Endwhile
Return Count
EndFunction

Void Function SayTrackFieldHeading ()
var
string CurrentTrackField
let CurrentTrackField = StringSegment (TrackFieldList, "|",TrackFieldIndex)
if CurrentTrackField == "name" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldName,OT_CONTROL_NAME)
elif CurrentTrackField == "type" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldType,OT_CONTROL_NAME)
elif CurrentTrackField == "blocks" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldBlocks,OT_CONTROL_NAME)
elif CurrentTrackField == "mute" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldMute,OT_CONTROL_NAME)
elif CurrentTrackField == "solo" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldSolo,OT_CONTROL_NAME)
elif CurrentTrackField == "arm" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldArm,OT_CONTROL_NAME)
elif CurrentTrackField == "pan" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldPan,OT_CONTROL_NAME)
elif CurrentTrackField == "volume" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldVolume,OT_CONTROL_NAME)
elif CurrentTrackField == "playdevice" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldPlayDevice,OT_CONTROL_NAME)
elif CurrentTrackField == "recdevice" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldRecDevice,OT_CONTROL_NAME)
elif CurrentTrackField == "midimap" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldMidiMap,OT_CONTROL_NAME)
elif CurrentTrackField == "fx" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldFX,OT_CONTROL_NAME) 
elif CurrentTrackField == "lock" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldLock,OT_CONTROL_NAME) 
elif CurrentTrackField == "EQHigh" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldEQHigh,OT_CONTROL_NAME) 
elif CurrentTrackField == "EQMid" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldEQMid,OT_CONTROL_NAME) 
elif CurrentTrackField == "EQLow" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldEQLow,OT_CONTROL_NAME) 
elif CurrentTrackField == "EQMode" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldEQMode,OT_CONTROL_NAME) 
elif CurrentTrackField == "Wet" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldWet,OT_CONTROL_NAME) 
elif CurrentTrackField == "Dry" then
SayUsingVoice(VCTX_PcCursor,msgTrackFieldDry,OT_CONTROL_NAME) 
Endif
EndFunction

Int Function IsSelected ()
if GetDECCEditboxValue(CidSelectTo) == CSCNull then
Return False
else
Return True
Endif
EndFunction

Void Function ContextMenuTrackField ()
var
string CurrentTrackField
let CurrentTrackField = StringSegment (TrackFieldList, "|", TrackFieldIndex)
SaveCursor()
if CurrentTrackField == "fx" then
JawsCursor()
MoveToWindow(FindDescendantWindow(appWindow,CidTrackFX))
RightMouseButton()
elif CurrentTrackField == "blocks" then
MoveToAndGetTrackBlockValue(0)
RouteJawsToInvisible()
RightMouseButton()
Endif
EndFunction

Void Function KeyPressedEvent (int NKey, string strKeyName, int nIsBraille, int nIsScript)
;Enable context menus for things like graphic buttons on toolbars
if nKey == K_Contextmenu then
PerformScript ContextMenu()
return
endIf

;Still speak "alt+tab" if key echo is set to characters and we are on the main area
if 		StringMatch(strKeyName, "LeftAlt+Tab") || StringMatch(strKeyName, "RightAlt+Tab") then
if (OldTypingEcho > 0) && (GetJCFOption (opt_typing_echo) == 0) then
SayUsingVoice(VCTX_PCCursor,"Alt tab",OT_JAWS_MESSAGE)
Endif
endIf
EndFunction

Script ContextMenu ()
if !isTrackTable() && !isWaveform() && !isToolbar(getFocus()) then
executeDefaultScript()
Return
Endif
if IsToolbar(GetFocus()) then tbButtonContextMenu()
elif isTrackTable() then
ContextMenuTrackField()
elif isWaveform() then
SaveCursor()
JawsCursor()
RouteJAWSToPc ()
RightMouseButton()
Endif
EndScript

Void Function updateEditorMode ()
;Has a mode been set at all yet?
if editorMode then
;yes.
;Lets backup the old editorMode value
let OldeditorMode = editorMode
Endif
;lets update the current mode value
let editorMode = getEditorMode()
EndFunction

Void Function SayEditorModeAndFile ()
if editorMode == editorModeWave then
SayUsingVoice(VCTX_PcCursor,FormatString(msgModeWave_L,GetFileName()),OT_DOCUMENT_NAME)
elif editorMode == editorModeTrack then
SayUsingVoice(VCTX_PcCursor,FormatString(msgModeTrack_L,GetFileName()),OT_DOCUMENT_NAME)
elif editorMode == editorModeCD then
SayUsingVoice(VCTX_PcCursor,msgModeCD,OT_DOCUMENT_NAME)
Endif
EndFunction

Void Function sayWaveform ()
SayUsingVoice(VCTX_PcCursor,msgWaveForm,OT_CONTROL_NAME)
SpeakSelectionOrEditCursor()
if GetTransport() == TransPlay then
Say("Playing:",OT_JAWS_MESSAGE)
PerformScript SayCurrentTime()
Endif
EndFunction

Void Function sayTrackTable ()
SayUsingVoice(VCTX_PcCursor,msgTrackTable,OT_CONTROL_GROUP_NAME)
SayTrackFieldHeading ()
SayTrackFieldValue ()
SpeakSelectionOrEditCursor()
if GetTransport() == TransPlay then
Say("Playing:",OT_JAWS_MESSAGE)
PerformScript SayCurrentTime()
Endif
EndFunction

Void Function SayTrackNavigation ()
if (NavigateTracksByVerbosity == CfgNavigateTracksByNumber) || (NavigateTracksByVerbosity == CfgNavigateTracksByNumberAndName) then
SayUsingVoice (VCTX_MESSAGE,GetTrackNumber(),OT_ITEM_NUMBER)
EndIf
if (NavigateTracksByVerbosity == CfgNavigateTracksByName) || (NavigateTracksByVerbosity == CfgNavigateTracksByNumberAndName) then
SayUsingVoice(VCTX_MESSAGE,GetWindowText(FindDescendantWindow(appWindow,CidTrackName),False),OT_POSITION)
EndIf
; If we're in the name field and navigating tracks by name, don't want it to be spoken twice.
if !(((NavigateTracksByVerbosity == CfgNavigateTracksByName) || (NavigateTracksByVerbosity == CfgNavigateTracksByNumberAndName)) &&
 (StringSegment (TrackFieldList, "|",TrackFieldIndex) == "name")) then
SayTrackFieldValue()
EndIf
EndFunction

Script SaySelection ()
SpeakSelectionOrEditCursor()
EndScript

String Function GetTrackEQMode ()
var
handle Window
let Window = FindDescendantWindow(appWindow,CidTrackEQMode)
if !IsWindowVisible(Window) then
Return msgHidden
Endif
SaveCursor()
JawsCursor()
MoveToWindow(Window)
Return StringSegment (GetWindowText(Window,False), "/", 2)
EndFunction

String Function ChangeTrackEQMode ()
var
handle Window
let Window = FindDescendantWindow(appWindow,CidTrackEQMode)
if !IsWindowVisible(Window) then
Return msgHidden
Endif
SaveCursor()
JawsCursor()
MoveToWindow(Window)
LeftMouseButton()
Pause()
Return GetTrackEQMode()
EndFunction

String Function GetSelectedTrackTab ()
if IsWindowVisible(FindDescendantWindow(appWindow,CidTrackEQMode)) then
Return "EQ"
elif IsWindowVisible(FindDescendantWindow(appWindow,CidTrackWet)) then
Return "Bus"
else
Return "Vol"
Endif
EndFunction

Function LoadTrackFieldList ()
var
string x
let x = GetSelectedTrackTab()
if x == "vol" then
let TrackFieldList = TrackFieldList_Vol
elif x == "eq" then
let TrackFieldList = TrackFieldList_EQ
elif x == "bus" then
let TrackFieldList = TrackFieldList_Bus
Endif 
EndFunction

Script TrackTabToggle ()
var
handle window,
int Cid,
string x
let x = GetSelectedTrackTab()
if x == "vol" then
let cid = CidTrackTab_EQ
elif x == "EQ" then
let Cid = CidTrackTab_Bus
elif x == "bus" then
let cid = CidTrackTab_Vol
else
Return
Endif 
SaveCursor()
JawsCursor()
let window = findDescendantWindow(appWindow,cidTrackDisp)
let window = findDescendantWindow(window,cid)
moveToWindow(window)
LeftMouseButton()
RouteInvisibleToJAWS ()
InvisibleCursor ()
Pause()
LoadTrackFieldList()
SayUsingVoice(VCTX_PcCursor,GetWindowText(GetCurrentWindow(),False),OT_SCREEN_MESSAGE)
let TrackFieldIndex = 1
Pause()
SayTrackFieldHeading()
SayTrackFieldValue()
EndScript

String Function MoveToAndGetTrackBlockValue (int ByRef NumBlocks)
var
handle window,
int count
let Window = FindDescendantWindow(appWindow,CidTrackFile)
RefreshWindow(Window)
if GetWindowText(window,false) == cscNull then
;SaveCursor()
;JawsCursor()
;MoveToWindow(Window)
return msgEmpty
EndIf
SaveCursor()
InvisibleCursor()
MoveToWindow(window)
let NumBlocks = 0
while GetCurrentWindow() == window
NextChunk()
let NumBlocks = NumBlocks + 1
EndWhile
if BlockNumber < 1 then ; if trying to move further left than is actually possible
let BlockNumber = 1
elif BlockNumber > NumBlocks then
let BlockNumber = NumBlocks
EndIf
let count = 1
MoveToWindow(window)
while count < BlockNumber
NextChunk()
let count = count + 1
EndWhile
return "\"" + GetChunk() + "\""
EndFunction

Void Function DragMouse (int direction, int NumPixelsSmall, int NumPixelsLarge)
LeftMouseButtonLock()
if direction == DirUp then
MouseUp(NumPixelsSmall)
elif direction == DirLargeUp then
MouseUp(NumPixelsLarge)
elif direction == DirDown then
MouseDown(NumPixelsSmall)
elif direction == DirLargeDown then
MouseDown(NumPixelsLarge)
elif direction == DirLeft then
MouseLeft(NumPixelsSmall)
elif direction == DirLargeLeft then
MouseLeft(NumPixelsLarge)
elif direction == DirRight then
MouseRight(NumPixelsSmall)
elif direction == DirLargeRight then
MouseRight(NumPixelsLarge)
EndIf
pause()
LeftMouseButtonLock()
EndFunction

String Function getDECCEditboxValue (int Cid)
Return GetWindowText(FindDescendantWindow(appWindow,Cid),False)
EndFunction


Int Function ConvertLeftToNegative (string Value)
var
int number,
string letter
let letter = StringSegment(value," ",1)
let number = StringToInt(StringSegment(Value," ",3))
if Letter == "l" then
let number = (number * -1)
Endif
return Number
EndFunction

Script SetTimeMark1 ()let TimeMark1 = GetTimeCurrent ()
SayUsingVoice(VCTX_MESSAGE,"1="+ConvertTimeUser(TimeMark1),OT_STATUS)
EndScript

Script SetTimeMark2 ()
let TimeMark2 = GetTimeCurrent()
SayUsingVoice(VCTX_MESSAGE,"2="+ConvertTimeUser(TimeMark2),OT_STATUS)
EndScript

Script SetSelectionToTimeMarks ()
ChangeDECCEditboxValue(CidSelectFrom,TimeMark1)
ChangeDECCEditboxValue(CidSelectTo,TimeMark2)
PerformScript SaySelection()
EndScript

Script SetTimeMarksToSelection ()
let TimeMark1=GetDECCEditboxValue(CidSelectFrom)
SayUsingVoice(VCTX_MESSAGE,"1="+ConvertTimeUser(TimeMark1),OT_STATUS)
let TimeMark2 = GetDECCEditboxValue(CidSelectTo)
SayUsingVoice(VCTX_MESSAGE,"2="+ConvertTimeUser(TimeMark2),OT_STATUS)
EndScript

Script SetViewToTimeMarks ()
ChangeDECCEditboxValue(CidViewFrom,TimeMark1)
ChangeDECCEditboxValue(CidViewTo,TimeMark2)
PerformScript SayView()
EndScript

Script SetTimeMarksToView ()
let TimeMark1=GetDECCEditboxValue(CidViewFrom)
SayUsingVoice(VCTX_MESSAGE,"1="+ConvertTimeUser(TimeMark1),OT_STATUS)
let TimeMark2 = GetDECCEditboxValue(CidViewTo)
SayUsingVoice(VCTX_MESSAGE,"2="+ConvertTimeUser(TimeMark2),OT_STATUS)
EndScript

String Function FindAndReplace (string Main, string Search, string Replacement)
var
int pos,
int mainLength,
int searchLength, 
string final
let mainLength = stringLength(main)
let searchLength = stringLength(search)
let pos = 1
while pos <= mainLength
if stringMatch(subString(main,pos,searchLength),search) then
let final = final + replacement
let pos = pos + searchLength
else
let final = final + subString(main,pos,1)
let pos = pos + 1
endIf
endWhile
return final
EndFunction

Void Function MoveToHighlightedCue ()
var
String cueName
SpeechOff()
let CueName = GetWindowName(FindDescendantWindow(GetDockingWindow("Cue List"),CidCueLabel))
RouteJawsToPC()
LeftMouseButton()
LeftMouseButton()
while !(getDockingWindowState(GetDockingWindow("Cue List")) == Closed)
Pause()
Endwhile
SpeechOn()
SayUsingVoice(VCTX_MESSAGE,FormatString(msgMovedTo,CueName),OT_STATUS)
if editorMode == editorModeTrack then
sayTrackTable()
elif editorMode == editorModeWave then
sayWaveform()
endIf
return
EndFunction

String Function ConvertTimeHMSF (string Time, int PositionalTimesVerbosity, int DescriptiveTimesVerbosity)
var
String hours,
string minutes,
string seconds,
string frames,
string NewValue
if PositionalTimesVerbosity then
if time == "00:00:00:00" then
return msgBeginning
;elif time == GetFileLength() then
;Return msgEnd
Endif
Endif
if DescriptiveTimesVerbosity then
if time == "00:00:00:00" then
Return "0frames"
Endif
let hours = IntToString(StringToInt(StringSegment(Time,":",1)))
 let minutes = IntToString(StringToInt(StringSegment(Time,":",2)))
let seconds = IntToString(StringToInt(StringSegment(Time,":",3)))
let frames = IntToString(StringToInt(StringSegment(Time,":",4)))
if hours != "00" then
let NewValue = hours+"hours"
Endif
if Minutes != "00" then
let NewValue = NewValue+Minutes+"minutes"
Endif
if Seconds != "00" then
let NewValue = NewValue+Seconds+"seconds"
Endif
if Frames != "00" then
let NewValue = NewValue+Frames+"frames"
Endif
return NewValue
Endif
Return time
EndFunction

String Function ConvertTimeDecimal (string time, int PositionalTimesVerbosity, int DescriptiveTimesVerbosity)
var
string hours,
string minutes,
string seconds,
int SecondsSeg,
string seconds1,
string seconds2

if PositionalTimesVerbosity && (time == "0:00.000") then
return msgBeginning
EndIf
if DescriptiveTimesVerbosity then
if time == "0:00.000" then
return "0seconds"
Endif
if StringSegment(time,":",3) then
; We have an hours field.
let hours = StringSegment(Time,":",1)
let minutes = StringSegment(Time,":",2)
let SecondsSeg = 3
else
let minutes = StringSegment(Time,":",1)
let SecondsSeg = 2
EndIf
let seconds = StringSegment(time,":",SecondsSeg)
let seconds1 = IntToString(StringToInt(StringSegment(seconds,".",1)))
let seconds2 = StringReverse(IntToString(StringToInt(StringReverse(StringSegment(seconds,".",2)))))
let seconds = seconds1
if seconds2 != "0" then
let seconds = seconds + "." + seconds2
EndIf
let time = ""
if hours != "" then
let time = hours+"hours"
Endif
if Minutes != "0" then
let time= time+Minutes+"minutes"
EndIf
if !StringMatch(Seconds,"0") then
let time = time+Seconds+"seconds"
EndIf
EndIf
Return time
EndFunction

String Function ConvertTimeBarsBeats (string time, int PositionalTimesVerbosity, int DescriptiveTimesVerbosity)
var
string bar,
string beat,
string beat1,
string beat2

if PositionalTimesVerbosity && (time == "1:1.00") then
return msgBeginning
EndIf
if DescriptiveTimesVerbosity then
if time == "0:0.00" then
return "0bars"
EndIf
let bar = StringSegment(time,":",1)
let beat = StringSegment(time,":",2)
let beat1 = IntToString(StringToInt(StringSegment(beat,".",1)))
let beat2 = StringReverse(IntToString(StringToInt(StringReverse(StringSegment(beat,".",2)))))
let beat = beat1
if beat2 != "0" then
let beat = beat + "." + beat2
EndIf
let time = bar+"bars"+beat+"beets"
EndIf
return time
EndFunction

string Function ConvertTimeSamples (string Time, int PositionalTimesVerbosity, int DescriptiveTimesVerbosity)
if PositionalTimesVerbosity && StringMatch(time, "0") then
return msgBeginning
EndIf
if DescriptiveTimesVerbosity then
let time = time + "samples"
EndIf
return time
EndFunction

String Function ConvertTimeUser (string time)
return ConvertTimeInternal(time,PositionalTimesVerbosity,DescriptiveTimesVerbosity)
EndFunction

Void Function tbAnnounceButton ()
var
string name,
int safetyCount
saveCursor()
jawsCursor()
let strLastToolTip = ""
moveTo(stringToInt(stringSegment(stringSegment(tb_buttonList,"|",tb_buttonIndex),",",1)),stringToInt(stringSegment(stringSegment(tb_buttonList,"|",tb_buttonIndex),",",2)))
if tb_buttonIndex == 1 then
nextChunk()
moveTo(stringToInt(stringSegment(stringSegment(tb_buttonList,"|",tb_buttonIndex),",",1)),stringToInt(stringSegment(stringSegment(tb_buttonList,"|",tb_buttonIndex),",",2)))
endIf
let safetyCount = 0
while !strLastTooltip && (safetyCount <= 20)
delay(1)
let safetyCount = safetyCount + 1
endWhile
restoreCursor()
IndicateControlType (WT_BUTTON,strLastTooltip)
EndFunction

Void Function tbNextButton ()
let tb_buttonIndex = tb_buttonIndex + 1
if tb_buttonIndex > tb_buttons then
let tb_buttonIndex = 1
endIf
EndFunction

Void Function tbPriorButton ()
let tb_buttonIndex = tb_buttonIndex - 1
if tb_buttonIndex == 0 then
let tb_buttonIndex = tb_buttons
endIf
EndFunction

Void Function MoveToMainArea ()
PutFocusToMainArea()
EndFunction

Void Function tbButtonContextmenu ()
SaveCursor()
JawsCursor()
moveTo(stringToInt(stringSegment(stringSegment(tb_buttonList,"|",tb_buttonIndex),",",1)),stringToInt(stringSegment(stringSegment(tb_buttonList,"|",tb_buttonIndex),",",2)))
pause()
RightMouseButton()
EndFunction

Void Function tbActivateButton ()
SaveCursor()
JawsCursor()
moveTo(stringToInt(stringSegment(stringSegment(tb_buttonList,"|",tb_buttonIndex),",",1)),stringToInt(stringSegment(stringSegment(tb_buttonList,"|",tb_buttonIndex),",",2)))
pause()
LeftMouseButton()
EndFunction

Void Function tbSetList ()
var
handle Window,
int OK
let tb_list = ""
let window = GetFirstChild(appWindow)
let OK = True
While OK 
if (GetWindowClass(Window) == "CoolToolb") &&
IsWindowVisible(Window)  then
let tb_list = FormatString("%1|%2",IntToString(Window),tb_list)
Endif
if GetNextWindow(Window) then
let Window = GetNextWindow(Window)
else
let OK = False
Endif
Endwhile
EndFunction

Void Function tbNextToolbar ()
SaveCursor()
;if !NumberOfStringsInString(tb_list,IntToString(tb_CurrentWindow)+"|") then
; Return
;Endif
let tb_index = tb_index + 1
if !StringSegment(tb_list,"|",tb_index) then
let tb_index = 0
Endif
if tb_index > 0 then
if tb_index == 1 then
let DisableMenuModeEvent = True
Endif
SetFocus(StringToHandle(StringSegment(tb_list,"|",tb_index)))
else
let DisableFocusChangedEvent = true
PutFocusToMainArea()
let DisableFocusChangedEvent = False
SpeechOff()
{alt}
Pause()
SpeechOn()
SayObjectActiveItem()
let DisableMenuModeEvent = False
Endif
EndFunction

Void Function tbPriorToolbar ()
SaveCursor()
;if !NumberOfStringsInString(tb_list,IntToString(CurrentToolbarWindow)+"|") then
; Return
;Endif
let tb_index = tb_index - 1
if tb_index == -1 then
let tb_index = NumberOfStringsInString(tb_list,"|")
Endif
if tb_index > 0 then
if tb_index == NumberOfStringsInString(tb_list,"|") then
let DisableMenuModeEvent = True
Endif
SetFocus(StringToHandle(StringSegment(tb_list,"|",tb_index)))
else
let DisableFocusChangedEvent = true
PutFocusToMainArea()
let DisableFocusChangedEvent = False
SpeechOff()
{alt}
Pause()
SpeechOn()
SayObjectActiveItem()
let DisableMenuModeEvent = False
Endif
EndFunction

Script CtrlTab ()
if IsToolbar(GetFocus()) ||
MenusActive() then
if MenusActive() then
tbSetList()
let tb_index=0
Endif
tbNextToolbar()
else
SwitchFiles()
Endif
EndScript

Script CtrlShiftTab ()
if IsToolbar(GetFocus()) ||
MenusActive() then
if MenusActive() then
tbSetList()
let tb_index=0
Endif
tbPriorToolbar()
else
SwitchFiles()
Endif
EndScript


Script MoveToTransportCtrls ()
var
handle Window
let Window = FindDescendantWindow(appWindow,CidTransportCtrls)
if !IsWindowDisabled(Window) then
SetFocus(Window)
else
SayUsingVoice(VCTX_MESSAGE,"Transport controls toolbar is not visible",OT_ERROR)
Endif
EndScript

Script MoveToZoomCtrls ()
var
handle Window
let Window = FindDescendantWindow(appWindow,CidZoomCtrls)
if Window then
setFocus(Window)
else
SayUsingVoice(VCTX_MESSAGE,"Zoom controls toolbar is not visible",OT_ERROR)
Endif
EndScript
											

Int Function IsToolbar (handle window)
if GetWindowClass(Window) == "CoolToolb" then
Return True
else
Return False
Endif
EndFunction


Int Function IsInSVCtrls (handle window)
if IsThisUnderWindow(Window,FindDescendantWindow(appWindow,CidSvCtrls)) then
Return True
else
Return False
Endif
EndFunction

HANDLE Function GetDockingWindow (string WindowName)
var
handle CurrentWindow
let CurrentWindow = appWindow
while (GetAppMainWindow(CurrentWindow) == AppWindow) 
if stringMatch(GetWindowName(CurrentWindow),WindowName) then
Return CurrentWindow
Endif
let CurrentWindow = GetPriorWindow(CurrentWindow)
Endwhile
EndFunction

Void Function WindowCreatedEvent (handle window, int LeftX, int TopY, int RightX, int BottomY)
var
int ControlID,
string WindowName
let WindowName = GetWindowName(window)
let ControlID = GetControlID(Window)
;If we don't know audition's main window, and it is created, then grab it.
;if !appWindow then
;if GetWindowClass(window) == classAudition then
;let appWindow = window
;Endif
;Endif

if WindowName == "Device Ordering Preferences" then
Pause()
Pause()
FocusChangedEvent(window,0)
Endif
WindowCreatedEvent (window, LeftX, TopY, RightX, BottomY)
EndFunction

HANDLE Function GetTimeRulerWindow ()
if editorMode == editorModeTrack then
return FindDescendantWindow(FindDescendantWindow(appWindow, cidTrackDisp), CidTimeRuler)
elif editorMode == editorModeWave then
return FindDescendantWindow(FindDescendantWindow(appWindow, cidWaveDisp), CidTimeRuler)
EndIf
EndFunction

String Function GetAssignedScript (string KeyName, string KeyMap)
return IniReadString ("common keys", KeyName, CSCNull,KeyMap)
EndFunction

Void Function tbInicialise (handle window)
let tb_buttonList = ""
let tb_buttons = 0
RefreshWindow(window)
;pause()
GraphicsEnumerate (window, "tbAddButton")
let tb_buttonIndex = 1
EndFunction

Void Function MenuModeEvent (handle WinHandle, int mode)
if DisableMenuModeEvent then
return
Endif
if mode > 0 then
RestoreTypingEcho()
;GlobalPrevMenuMode hasn't been updated yet, so have to use GlobalMenuMode
if GlobalMenuMode == Menu_Inactive then 
;This is so speech is interupted when menu bar is activated whilst trying to say really long things like the current time
StopSpeech()
Endif
if WinHandle == appWindow then
tbSetList()
let tb_index=0
Endif
Endif
if mode == MENU_INACTIVE then

stopSpeech()
endIf

MenuModeEvent(WinHandle, mode)
;This next bit of code makes sure that the Main Area is spoken when leaving an active menu.
;For some odd reason, when going from the menu bar in to an active menu, WaveCtrlClass gets focus
if (Mode == Menu_Inactive) && (GlobalPrevMenuMode == Menu_active) then
if isTrackTable() then
sayTrackTable()
elif isWaveform() then
sayWaveform()
endIf
Endif
EndFunction

String Function GetTimeFormatType ()
var
string label,
string types,
int count,
int total
let types = "4|fps|smpl|hms|bpm"
SaveCursor()
InvisibleCursor()
MoveToWindow(GetTimeRulerWindow())
JawsHome()
let label = getChunk()
let total = stringToInt(stringSegment(types,"|",1))
let count = 1
while (count <= total)
if stringContains(label,stringSegment(types,"|",count+1)) then
return stringSegment(types,"|",count+1)
endIf
let count = count + 1 
endWhile
EndFunction

Void Function configureAuditionWindows ()
var
handle TempWindow

;Is the Cue list window open, but lost focus?
if (getDockingWindowState(GetDockingWindow("Cue List")) == Lost) then
SayUsingVoice(VCTX_MESSAGE,"Closing Cue List window",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_command,command_toggleCueListWindow)
Endif

;Close that silly Track properties window
if (getDockingWindowState(GetDockingWindow("Track Properties")) == Lost) then
SayUsingVoice(VCTX_MESSAGE,"Closing Track Properties window",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_command,command_toggleTrackPropertiesWindow)
Endif 

;Close the Frequency Analysis window
if (getDockingWindowState(GetDockingWindow("Frequency Analysis")) == Lost) then
SayUsingVoice(VCTX_MESSAGE,"Closing Frequency Analysis window",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_command,command_toggleFrequencyAnalysisWindow)
Endif 

;Close the Session Properties window (audition 1.0)
if (getDockingWindowState(GetDockingWindow("Session Info")) == Lost) then
SayUsingVoice(VCTX_MESSAGE,"Closing Session Properties window",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_command,command_toggleSessionPropertiesWindow)
Endif 

;Close the Session Properties window (audition 1.5)
if (getDockingWindowState(GetDockingWindow("Session Properties")) == Lost) then
SayUsingVoice(VCTX_MESSAGE,"Closing Session Properties window",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_command,command_toggleSessionPropertiesWindow)
Endif 

;Close the Play List window
let tempWindow = getDockingWindow("Play List")
if getDockingWindowState(tempWindow) == lost then
SayUsingVoice(VCTX_MESSAGE,"Closing Play List window",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_command,command_togglePlayListWindow)
Endif 

;Close the Track EQ window
if (getDockingWindowState(GetDockingWindow("Track Equalizers")) == Lost) then
SayUsingVoice(VCTX_MESSAGE,"Closing Track EQ window",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_command,command_toggleTrackEQWindow)
endIf

;Fix the mixers window
if (getDockingWindowState(GetDockingWindow("Mixers")) == Lost) then
SayUsingVoice(VCTX_MESSAGE,"Closing Track Properties window",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_command,command_toggleMixersWindow)
Endif

;Fix the Organizer window
let tempWindow = getDockingWindow("Organizer")
if tempWindow && (getWindowTypecode(getFocus()) != WT_DIALOG) then
SayUsingVoice(VCTX_MESSAGE,"Closing Organizer window",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_command,command_toggleOrganizerWindow)
endIf

;Get rid of the Place Holder window
let TempWindow = GetDockingWindow("Adobe Audition (dock this anywhere you like)")
if TempWindow then
SayUsingVoice(VCTX_MESSAGE,"Closing Place Holder window",OT_JAWS_MESSAGE)
sendMessage(tempWindow,WM_CLOSE)
Endif

;Make sure the selection / view controls are always visible
if !IsWindowVisible(FindDescendantWindow(appWindow,cidSVCtrls)) then
SayUsingVoice(VCTX_MESSAGE,"Making the Selection / View controls visible",OT_JAWS_MESSAGE)
let tempWindow = getFocus()
sendMessage(commandWindow,WM_command,command_toggleSVCtrls)
Delay (1, true)
setFocus(tempWindow)
Endif

;Make sure the status bar is visible
let tempWindow = getFirstChild(appWindow)
if !isWindowVisible(tempWindow) then
SayUsingVoice(VCTX_MESSAGE,"Making the Status bar visible",OT_JAWS_MESSAGE)
sendMessage(commandWindow,WM_COMMAND,command_toggleStatusbar)
endIf
EndFunction

Int Function handleCustomWindows (handle window)
var
handle realWindow,
int processed
let processed = false
if disableFocusChangedEvent then
return true
endIf

if window == globalPrevFocus then
return true
 endIf

let realWindow = getRealWindow(window)

;Another chance to grab audition's main window
if !appWindow then
if(getWindowClass(realWindow) == classAudition) then
let appWindow = realWindow
let commandWindow = getFinalWindow(getFirstChild(appWindow))
endIf
endIf

;Focus should never land directly on the audition main window
if window == appWindow 
&& !menusActive() then 
scheduleFunction("putFocusToMainArea",0)
return true
endIf

;Process new text
ScheduleFunction ("ProcessNewText", 0)

;Correct the menu mode after dropping out of a toolbar
if !MenusActive() && IsToolbar(globalPrevFocus) && !IsToolbar(window) && DisableMenuModeEvent then
let DisableMenuModeEvent = false
MenuModeEvent(globalPrevFocus, MENU_INACTIVE)
Endif

updateEditorMode()

if (editorMode != OldeditorMode) then
SayeditorModeAndFile()
Endif

;Speak the splash screen
if (getWindowClass(Window) == ClassSplash) &&
!MenusActive() then
IndicateControlType (0,"", msgSplash)
return true
endIf

;Prompts for the Settings dialog
if stringMatch(GetWindowName(GetRealWindow(window)),"Settings") then
if GetDialogPageName () == "System" then
if(getControlID(window) == cidSettingsSystemBufferSize) then
IndicateControlType (WT_EDIT, "Buffer size (in seconds):")
return true
endIf
if(getControlID(window) == cidSettingsSystemNumBuffers) then
IndicateControlType (WT_EDIT, "Number of buffers:")
return true
endIf
if(getControlID(window) == cidSettingsSystemPreviewBufferSize) then
IndicateControlType (WT_EDIT, "Pre	view buffer size (in milliseconds):")
return true
endIf
if(getControlID(window) == cidSettingsSystemWaveCacheSize) then
IndicateControlType (WT_EDIT, "Wave cache size (in K bytes):")
return true
endIf
if GetControlID(Window) == CidSettingsSystemBrowseForPrimaryTempFolder then
IndicateControlType (WT_BUTTON, msgSettingsSystemBrowseForPrimaryTempFolder, "")
return true
endIf
if GetControlID(Window) == CidSettingsSystemBrowseForSecondaryTempFolder then
IndicateControlType (WT_BUTTON, msgSettingsSystemBrowseForSecondaryTempFolder, "")
return true
endIf
Endif ;for system page
if GetDialogPageName () == "Data" then
if(getControlID(window) == cidSettingsDataCrossfadeLength) then
IndicateControlType (WT_EDIT, "Crossfade length (in milliseconds):")
return true
endIf
endIf ;for data page 
Endif ;for Settings dialog

if(getWindowClass(getRealWindow(window)) == "floatingDockWndClass") then
if(getWindowName(getRealWindow(window)) == "Cue List") then
if(getControlID(window) == CidCueList) then
sayString("cues")
endIf
endIf ;for cue list
if GetWindowName(GetRealWindow(window)) == "Track Properties" then
	if GetControlID(Window) == CidTrackPropertiesVolume then
IndicateControlType (WT_EDIT, "Volume:", getWindowText(GetFirstChild(window),false))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesPan then
IndicateControlType (WT_EDIT, "Pan:", getWindowText(GetFirstChild(window),false))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesChannels then
IndicateControlType (WT_COMBOBOX, "Channels")
return true
endIf
if GetControlID(Window) == CidTrackPropertiesBitDepth then
IndicateControlType (WT_COMBOBOX, "Bit depth")
return true
endIf
if GetControlID(Window) == CidTrackPropertiesFX then
IndicateControlType (WT_BUTTON, "FX")
return true
endIf
if GetControlID(Window) == CidTrackPropertiesWet then
IndicateControlType (WT_spinbox, "Wet",getEndValue(getWindowText(window,false)))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesDry then
IndicateControlType (WT_spinbox, "Dry",getEndValue(getWindowText(window,false)))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesEQLow then
IndicateControlType (WT_SPINBOX, "Low EQ band:",getEndValue(getWindowText(window,false)))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesEQMid then
IndicateControlType (WT_SPINBOX, "Mid EQ band:",getEndValue(getWindowText(window,false)))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesEQHigh then
IndicateControlType (WT_SPINBOX, "High EQ band:",getEndValue(getWindowText(window,false)))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesMute then
IndicateControlType (WT_BUTTON, "Mute")
IndicateControlState (wt_button, getDECCState(window))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesSolo then
IndicateControlType (WT_BUTTON, "Solo")
IndicateControlState (wt_button, getDECCState(window))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesRecord then
IndicateControlType (WT_BUTTON, "Record")
IndicateControlState (wt_button, getDECCState(window))
return true
endIf
if GetControlID(Window) == CidTrackPropertiesOutDLG then
IndicateControlType (WT_BUTTON, "Output devices...")
return true
endIf
if GetControlID(Window) == CidTrackPropertiesInDLG then
IndicateControlType (WT_BUTTON, "Input devices...")
IndicateControlState (wt_button, getDECCState(window))
return true
endIf

endIf ;for track properties
IF getWindowName(getRealWindow(window)) == "Track Equalizers" then
if getControlID(window) == cidTrackEqualizersDBLow then
IndicateControlType (wt_spinbox, "Low band DB", getEndValue(getWindowText(window,false)))
return true
elif getControlID(window) == cidTrackEqualizersDBMid then
IndicateControlType (wt_spinbox, "Mid band DB", getEndValue(getWindowText(window,false)))
return true
elif getControlID(window) == cidTrackEqualizersDBHigh then
IndicateControlType (wt_spinbox, "High band DB", getEndValue(getWindowText(window,false)))
return true
elif getControlID(window) == cidTrackEqualizersHZLow then
IndicateControlType (wt_spinbox, "Low band HZ", getEndValue(getWindowText(window,false)))
return true
elif getControlID(window) == cidTrackEqualizersHZMid then
IndicateControlType (wt_spinbox, "Mid band HZ", getEndValue(getWindowText(window,false)))
return true
elif getControlID(window) == cidTrackEqualizersHZHigh then
IndicateControlType (wt_spinbox, "High band HZ", getEndValue(getWindowText(window,false)))
return true
endIf
endIf
Endif ;For floating dock windows

if getRealWindow(window) == appWindow then ;under audition main window
if(getWindowTypecode(getParent(window)) == WT_DIALOG) then ;organizers window
if(getWindowTypecode(getParent(globalPrevFocus)) != WT_DIALOG) 
|| (globalPrevReal != getRealWindow(window)) then
IndicateControlType (WT_DIALOG, "Organizer")
endIf ;for reporting Organizers window
endIf ;for organizers window

;Sometimes focus can land on some strange fader control, this is bad so we
;just move our focus back to the Main Area (wave control or track control, depending on the Audition mode)
if GetControlID(window) == CidCepFader then
PutFocusToMainArea()
return true
endIf ;for bad cepFader

;Speak all the selection / view controls fields
if IsThisUnderWindow(window,FindDescendantWindow(appWindow,CidSVCtrls)) then
;Speak selection/view group box if this is first time
if !isThisUnderWindow(GlobalPrevFocus,FindDescendantWindow(appWindow,cidSVCtrls)) then
IndicateControlType (WT_GROUPBOX, msgSVCtrls," ")
Endif
if GetControlID(GetParent(Window)) == CidSelectFrom then
IndicateControlType (WT_EDIT, msgSVCtrlsSelectFrom)
return true
endIf
if GetControlID(GetParent(Window)) == CidSelectTo then
IndicateControlType (WT_EDIT, msgSVCtrlsSelectTo)
return true
endIf
if GetControlID(GetParent(Window)) == CidSelectLength then
IndicateControlType (WT_EDIT, msgSVCtrlsSelectLength)
return true
endIf
if GetControlID(GetParent(Window)) == CidViewFrom then
IndicateControlType (WT_EDIT, msgSVCtrlsViewFrom)
return true
endIf
if GetControlID(GetParent(Window)) == CidViewTo then
IndicateControlType (WT_EDIT, msgSVCtrlsViewTo)
return true
endIf
if GetControlID(GetParent(Window)) == CidViewLength then
IndicateControlType (WT_EDIT, msgSVCtrlsViewLength)
return true
endIf
endIf ;for SVCtrls

;A Toolbar 
if IsToolbar(window) then
RestoreTypingEcho()
tbInicialise(window)
IndicateControlType (WT_TOOLBAR,GetObjectName())
if isWindowObscured(window) then
SayFormattedMessage (OT_MESSAGE, msgToolbarObscured_L, msgToolbarObscured_S)
endIf
tbAnnounceButton()
return true
endIf

if isTrackTable() then
if globalPrevReal != appWindow then
SayeditorModeAndFile()
Endif

configureAuditionWindows()

;Grab the trackNumber window handle and also Zoom the track view in for best results.
;Find out the window handle of the track numbers ruler (can't jump to it by control for some odd reason)
let WndTrackNumber = GetTrackNumberWindow()
;Takes a while for Audition to draw the track number ruler
SpeechOff()
Pause()
SpeechOn()
ZoomTrackview ()
;Setup the column headings for the track table
LoadTrackFieldList()
sayTrackTable()
;Set typing echo to silent, but save the user's preference.
SilenceTypingEcho ()
return true
elif isWaveform() then
if globalPrevReal != appWindow then
SayeditorModeAndFile()
Endif

configureAuditionWindows()

;Set typing echo to silent, but save the user's preference.
SilenceTypingEcho ()
sayWaveform()
return true
endIf
endIf ;for under appWindow

if getWindowClass(window) == "dragEditCtrlClass" then
IndicateControlType (getDECCType(window), getDECCName(window))
IndicateControlState (getDECCType(window), getDECCState(window))
return true
endIf

return handleCustomWindows(window)
EndFunction

Int Function handleCustomRealWindows (handle window)
var
string class,
int processed
let processed = false
if disableFocusChangedEvent then
return true
endIf

let class = getWindowClass(window)

;Another chance to grab audition's main window
if !appWindow then
if(class == classAudition) then
let appWindow = window
endIf
endIf

;Correct the menu mode after dropping out of a toolbar
if !MenusActive() && IsToolbar(globalPrevFocus) && !IsToolbar(window) && DisableMenuModeEvent then
let DisableMenuModeEvent = false
MenuModeEvent(globalPrevFocus, MENU_INACTIVE)
Endif

updateEditorMode()

if (editorMode != OldeditorMode) then
SayeditorModeAndFile()
Endif

if(class == "floatingDockWNDClass") then
IndicateControlType (WT_DIALOG,getWindowName(window))
setFocus(getFirstChild(getFirstChild(window)))
return true
endIf

return handleCustomRealWindows(window)
EndFunction

String Function getTrackfieldState (int state)
if state == CTRL_NONE then
return msgNo
elif state == CTRL_PRESSED then
return msgYes
elif state == CTRL_DISABLED then
return msgDisabled
elif state == CTRL_GRAYED then
return msgHidden
endIf 
EndFunction

Script test ()
var
handle window,
int count,
int res,
int x,
int y
;let window = getCurrentWindow()
let window = commandWindow
let x = getWindowLeft(window)+(getWindowRight(window)-getWindowLeft(window))/2
let y = getWindowTop(window)+(getWindowBottom(window)-getWindowTop(window))/2 
let count = 0
while count <= 8000
let res = sendMessage(window,wm_user+count)
if res then
sayString("message "+intToString(count)+" has value "+intToString(res))
endIf
let count = count + 1
endWhile
sayString("done")
EndScript


Script test2 ()

EndScript

Int Function getDECCState (handle window)
var
int controlState,
int WMBits
if !IsWindowVisible(Window) then
let controlState = controlState + CTRL_GRAYED
elif IsWindowDisabled(Window) then
let controlState = controlState + CTRL_DISABLED
Endif
let WMBits = sendMessage(window,WM_USER+DECC_WM_state,0,0)
if WMBits & DECC_state_pressed then
let controlState = controlState + CTRL_PRESSED
endIf
return controlState
EndFunction

Int Function getDECCType (handle window)
var
int styleBits		
let styleBits = getWindowStyleBits(window)
;if !stringMatch(getWindowName(window),getWindowText(window,false)) then
;return WT_spinBox
if styleBits == DECC_ws_button then
return WT_button
elif styleBits == DECC_ws_edit then
return WT_edit
elif styleBits == DECC_ws_static then
return WT_static
elif styleBits == DECC_ws_commandButton then
return WT_dialog
else
return WT_unknown
endIf
EndFunction

String Function getDECCName (handle window)
return getWindowName(window)
EndFunction

String Function getDECCValue (handle window)
var
string name,
string value
let name = getWindowName(window)
let value = getWindowText(window,false)
if !stringMatch(name,value) then
return value
endIf
EndFunction

Script auditionNextTrack ()
if !isTrackTable() then 
executeDefaultScript()
return
endIf
sendMessage(commandWindow,wm_command,command_trackDown)
pause()
sayTrackNavigation()
EndScript

Script auditionPriorTrack ()
if !isTrackTable() then 
executeDefaultScript()
return
endIf
sendMessage(commandWindow,wm_command,command_trackUp)
pause()
sayTrackNavigation()
EndScript

Script auditionMoveCursorToBeginningOfFile ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
sendMessage(commandWindow,wm_command,command_beginning)
pause()
speakSelectionOrEditCursor()
EndScript

Script auditionMoveCursorToEndOfFile ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
sendMessage(commandWindow,wm_command,command_end)
pause()
speakSelectionOrEditCursor()
EndScript

Script auditionZoomInHorizontally ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
sendMessage(commandWindow,wm_command,command_zoomInHorizontally)
pause()
SayUsingVoice(VCTX_MESSAGE,ConvertTimeInternal(GetDECCEditboxValue(CidViewLength),False,PositionalTimesVerbosity),OT_MESSAGE)
EndScript

Script auditionZoomOutHorizontally ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
sendMessage(commandWindow,wm_command,command_zoomOutHorizontally)
pause()
SayUsingVoice(VCTX_MESSAGE,ConvertTimeInternal(GetDECCEditboxValue(CidViewLength),False,PositionalTimesVerbosity),OT_MESSAGE)
EndScript

Script auditionSelectAll ()
if !isTrackTable() && !isWaveform() then 
executeDefaultScript()
return
endIf
sendMessage(commandWindow,wm_command,command_selectAll)
pause()
SpeakSelectionOrEditCursor ()
EndScript

HANDLE Function getFinalWindow (handle window)
while getNextWindow(window)
let window = getNextWindow(window)
endWhile
return window
EndFunction

Function executeDefaultScript ()
var
string scriptName,
string keyboardType,
string keyName
let keyName = GetCurrentScriptKeyName ()
let scriptName = IniReadString ("common keys", keyName, "", "default.jkm")
if !scriptName then
let keyboardType = IniReadString ("options", "keyboardType", "desktop", getJawsDirectory()+"/user.ini")
let scriptName = IniReadString (keyboardType, keyName, "", "default.jkm")
endIf
if scriptName then
scheduleFunction("$"+scriptName,0)
else
typeCurrentScriptKey()
endIf
EndFunction

Void Function moveTrackFieldValue (int motion)
var
string currentTrackField,
handle window,
handle trackView
let trackView = findDescendantWindow(appWindow,cidTrackDisp)
let currentTrackField = getCurrentTrackField()
if currentTrackField == "pan" then
let window = findDescendantWindow(trackView,cidTrackPan)
scrollDECCSpinBox (window, motion)
elif currentTrackField == "volume" then
let window = findDescendantWindow(trackView,cidTrackVolume)
scrollDECCSpinBox (window, motion)
elif currentTrackField == "EQLow" then
let window = findDescendantWindow(trackView,cidTrackEQLow)
scrollDECCSpinBox (window, motion)
elif currentTrackField == "EQMid" then
let window = findDescendantWindow(trackView,cidTrackEQMid)
scrollDECCSpinBox (window, motion)
elif currentTrackField == "EQHigh" then
let window = findDescendantWindow(trackView,cidTrackEQHigh)
scrollDECCSpinBox (window, motion)
elif currentTrackField == "wet" then
let window = findDescendantWindow(trackView,cidTrackWet)
scrollDECCSpinBox (window, motion)
elif currentTrackField == "dry" then
let window = findDescendantWindow(trackView,cidTrackDry)
scrollDECCSpinBox (window, motion)
elif currentTrackField == "blocks" then
if (motion == sb_lineUp) || (motion == sb_pageUp)  then
let blockNumber = blockNumber + 1
elif (motion == sb_lineDown) || (motion == sb_pageDown) then
let blockNumber = blockNumber - 1
endIf
endIf
sayTrackFieldValue()
EndFunction

Script increaseTrackFieldValueSmall ()
if !isTrackTable() then
executeDefaultScript()
return
endIf
if isSameScript() then
pause()
endIf
moveTrackFieldValue(sb_lineUp)
EndScript

Script increaseTrackFieldValueLarge ()
if !isTrackTable() then
executeDefaultScript()
return
endIf
if isSameScript() then
pause()
endIf
moveTrackFieldValue(sb_pageUp)
EndScript

Script decreaseTrackFieldValueSmall ()
if !isTrackTable() then
executeDefaultScript()
return
endIf
if isSameScript() then
pause()
endIf
moveTrackFieldValue(sb_lineDown)
EndScript

Script decreaseTrackFieldValueLarge ()
if !isTrackTable() then
executeDefaultScript()
return
endIf
if isSameScript() then
pause()
endIf
moveTrackFieldValue(sb_PageDown)
EndScript

String Function getCurrentTrackField ()
return StringSegment (TrackFieldList, "|", TrackFieldIndex)
EndFunction


String Function getEndValue (string value)
let value = stringReverse(value)
let value = StringSegment (value, " ", 1)
let value = stringReverse(value)
return value
EndFunction


Int Function tbAddButton (int left, int top, int right, int bottom)
var
int x,
int y
let x =left+(right-left)/2
let y = top+(bottom-top)/2
let tb_buttonList = formatString("%1%2,%3|",tb_buttonList,intToString(x),intToString(y))
let tb_buttons = tb_buttons + 1
return true
EndFunction
